/*
% (C)2012-13 Syama Sundar Rangapuram, Thomas Buehler and Matthias Hein 
% Machine Learning Group, Saarland University, Germany
% http://www.ml.uni-saarland.de
*/
#include <math.h>
#include "mex.h"
#include "matrix.h"
#include <time.h>

double* ProjKSimplex(double*, int,int);
void OptSimplex(double* x,double eta, double D,int len);
int cmp_pointers(const void *wert1, const void *wert2);



void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[]) {
  // Test number of parameters.
  if (nrhs != 9 || nlhs != 8) {
    mexWarnMsgTxt("Usage: [X,rval,PrimalObj,DualObj,iter,Xbest,PrimalObjBest,DualObjBest]=solveInnerProblem(W,Y,rval,MAXITER,EPS,FourTimesMaxSumSquaredWeights,lambda,eps2)");
    return;
  }
  
  // get important parameters
  int rows = (int)mxGetM(prhs[0]); // number of rows of W
  int cols = (int)mxGetN(prhs[0]); // number of columns of W (should be the same)
  int len  = (int)mxGetM(prhs[1]); // the desired output
  int lenrval = (int)mxGetM(prhs[2]); // rval
    
  if(!mxIsSparse(prhs[0])) { mexWarnMsgTxt("Matrix is not sparse");}
  
  if(rows!=cols){
    mexWarnMsgTxt("Sparse matrix is not square");
    return;
  }

  if(rows!=len){
    mexWarnMsgTxt("Length of the vector is not the same as the number of the rows of the sparse matrix");
    return;
  }
    
  // Create output array and compute values
  double* sr = mxGetPr(prhs[0]);     // get values
  mwIndex* irs = mxGetIr(prhs[0]);   // get row
  mwIndex* jcs = mxGetJc(prhs[0]);   // get columns
  
  double* Xobs = mxGetPr(prhs[1]);		// Y
  double* rval = mxGetPr(prhs[2]);     // corresponds to y

  int MAXITER = (int) mxGetScalar(prhs[3]); 
  double EPS = mxGetScalar(prhs[4]); //for criterion fabs(dualval)<EPS
  double MaxSumSquaredWeights = mxGetScalar(prhs[5]); 
  double lambda = (double)mxGetScalar(prhs[6]); // lambda
  double eps2 = mxGetScalar(prhs[7]);  // for criterion gap/fabs(fval)<eps2
//  double eta = mxGetScalar(prhs[8]); 

  int MAXITER_temp = (int) mxGetScalar(prhs[8]); 
    
  if(MaxSumSquaredWeights<=0){
	  mexWarnMsgTxt("Lipschitz constant has to be positive");
    return;
  }

//MaxSumSquaredWeights=MaxSumSquaredWeights*100;
  double *X;      /* output matrix */
  plhs[0] = mxCreateDoubleMatrix(len,1,mxREAL); /* create the output vector */
  plhs[1] = mxCreateDoubleMatrix(lenrval,1,mxREAL); /* create the output dual variable */
  plhs[2] = mxCreateDoubleMatrix(1,1,mxREAL); /* create the output objective value */
  plhs[3] = mxCreateDoubleMatrix(1,1,mxREAL); /* create the final iteration value */
  
  plhs[4] = mxCreateDoubleMatrix(1,1,mxREAL); /* create the final iteration value */
  
   plhs[5] = mxCreateDoubleMatrix(len,1,mxREAL); /* create the output vector (best primal objective)*/
  plhs[6] = mxCreateDoubleMatrix(1,1,mxREAL); /* create the output objective value (best primal objective)*/
  plhs[7] = mxCreateDoubleMatrix(1,1,mxREAL); /* create the dual objective value (best primal objective)*/
  
  X = mxGetPr(plhs[0]); 
  double* Z = mxGetPr(plhs[1]);
  double* PrimalObj = mxGetPr(plhs[2]);
  double* DualObj = mxGetPr(plhs[3]);
  double* FinalIter = mxGetPr(plhs[4]);
  double* Xbest = mxGetPr(plhs[5]); 
  double* PrimalObjBest = mxGetPr(plhs[6]);
  double* DualObjBest = mxGetPr(plhs[7]);

  int counter=0,i,j,iter=0;
  double alpha,beta,factor;
  double dummy,normD,Fval;
  double* dummyPointer;
  double lamByL=lambda/MaxSumSquaredWeights;

  double* D =new double[len];
  double* Dproj =new double[len];
  double* Dproj2 =new double[len];
  double* DprojBest =new double[len];
  double* pval    = new double[lenrval];
  double* pvalold = new double[lenrval];
  for(i=0; i<lenrval; i++) { pval[i]=0; }
  for(i=0; i<lenrval; i++) { pvalold[i]=0; }
  double temp,primalVal,primalVal2,dualValBest,primalVal_orig;
  
  double C=0;
  for(i=0;i<len;i++){C+=Xobs[i]*Xobs[i];}
  C=0.5*C;
  
  double primalValBest=0;double max_f;
  bool computeObjective;

  for(i=0; i<len; i++) DprojBest[i]=0;

  clock_t start,ende;
  float time1=0,time2=0, time3=0,time4=0;

  //int MAXITER_temp=50;
 
  Fval=EPS+1;
  while(iter<MAXITER)
  {
    // defines strategy when to check if converged
	if (iter==MAXITER-1 || iter==MAXITER_temp)     {
        computeObjective=true;
    } else {   
		computeObjective =false;
    }
      
    // p corresponds to x, r correponds to y
    // exchange pval and pvalold
	dummyPointer=pval; pval=pvalold; pvalold=dummyPointer;
 
	// initialize X with zeros 
    for(i=0; i<len; i++) { X[i]=0; }

    // A alpha
	dummy=0; counter=0;
    for(j=0; j<cols; j++) 
    {   
	   for(i=0; i<jcs[j+1]-jcs[j]; i++)
	   {  
         dummy = sr[counter]*rval[counter];
	     X[j] -= dummy;
	     X[irs[counter]] += dummy;
         counter++;
	   }
	} 
 
    //D = Y- lambda Aalpha
    for(i=0; i<len; i++) { 
        D[i]=Xobs[i]-lambda*X[i];
        
		//Dproj[i]=(Xobs[i]-lambda*X[i])/eta; 
        //if (Dproj[i]<0) {Dproj[i]=0;}
        
        
        if (D[i]<0) Dproj[i]=0;
        else Dproj[i]=D[i];
        
    }
    
    //OptSimplex(Dproj,eta, C,len);
    //Dproj = ProjKSimplex(Dproj,len,1);
    
//     for (i=0;i<len;i++){
//         Dproj[i]=(D[i]-Dproj[i]);
//         if (Dproj[i]<0) Dproj[i]=0;
//     }
/*    
    
    for (i=0;i<8;i++)
        mexPrintf("%.5g ",Dproj[i]);
    mexPrintf("\n");*/
    
    
    //Dproj=OptSimplex(Dproj,eta, C,len);
/*	
    for (i=0;i<8;i++)
        mexPrintf("%.5g ",Dproj[i]);
    mexPrintf("\n");
    
    for(i=0; i<len; i++) { 
		D[i]=Xobs2[i]-lambda*X[i]; 
        temp=D[i]; if (temp<0) {temp=0;}
        Dproj3[i]=temp;
    }
    
    
    for (i=0;i<8;i++)
        mexPrintf("%.5g ",Dproj3[i]);
    mexPrintf("\n");
    
    Dproj3=OptSimplex(Dproj3,eta, C,len);
    
    for (i=0;i<8;i++)
        mexPrintf("%.5g ",Dproj3[i]);
    mexPrintf("\n");*/
    
    
    

/*    
    if(computeObjective)
    {
    // Dproj is f
    // compute primal objective (before making new step)
    // 0.5* ||Y-f||^2
		primalVal_orig=0;
		for (i=0;i<len; i++)
		{
            primalVal_orig-=Dproj[i]*Xobs[i];
		}
        
        max_f=0;
        for (i=0;i<len;i++)
        {
            if (Dproj[i]>max_f)
            	max_f=Dproj[i];
        }
        primalVal =primalVal_orig+2*eta*max_f;
    
	
		primalVal=primalVal_orig+C;
		temp=0;
		for (i=0;i<len; i++)
		{
            temp+=Dproj[i]* Dproj[i];
		}
		primalVal+=0.5*temp;
	}
    
	
   
   */
   
  
   
    
    
    // primalval2 = lambda*sum(w_ij|f_i-f_j|)
    // update pval (=x) and rval (=y)
    counter=0;
    
	//start = clock();
	//ende = clock();
	//time2+=(float) (ende-start);
    
    //primalVal2=0;
    factor=(double) iter/(iter+3);
    for(j=0; j<cols; j++) 
    {   
       alpha=Dproj[j];
	   for(i=0; i<jcs[j+1]-jcs[j]; i++)
	   {             
	      temp=sr[counter]*( Dproj[irs[counter]] - alpha);
        //  if (computeObjective){
		//	primalVal2+=fabs(temp);
		//  }
          // update of pval
          beta=rval[counter] + lamByL*temp;
          // projection onto l_inf-cube 
          if(beta>1) beta=1;
	      else if(beta<-1) beta=-1;
		  pval[counter]=beta;
		  // update of rval
          rval[counter] = beta + factor*(beta-pvalold[counter]);
          counter++;
	   }	  
    }
    
    
    
    
    
    
    
    
    
 /*  
    if (computeObjective)
    {
       primalVal2_b=primalVal+lambda*primalVal2;
	   primalVal_orig_b=primalVal_orig+lambda*primalVal2;
       normD = 0; 
       for(i=0; i<len; i++) { 
        //    normD+= (0.5 *Dproj2[i]- D[i])*Dproj2[i];
			normD-= 0.5 *Dproj[i]*Dproj[i];
		}
		Fval_b = normD + C;
    primalVal_orig=primalVal_orig_b;
   Fval=Fval_b;
    }
   
   */
	
	if (computeObjective)
	{
		start=clock();
       
		// compute dual val
		for(i=0; i<len; i++) { X[i]=0; }

		dummy=0; counter=0;
		for(j=0; j<cols; j++) 
		{   
			for(i=0; i<jcs[j+1]-jcs[j]; i++)
			{  
				dummy = sr[counter]*pval[counter];
		        X[j] -= dummy;
				X[irs[counter]] += dummy;
				counter++;
			}
		} 
		// compute objective with pval
		//for(i=0; i<len; i++) { 
         //   Dproj2[i]=(Xobs[i]-lambda*X[i])/eta; 
        //    if (Dproj2[i]<0) {Dproj2[i]=0;}
            //else Dproj2[i]=D[i];
        //}
		        
        //Dproj2=OptSimplex(Dproj2,eta, C,len);
        //OptSimplex(Dproj2,eta, C,len);
        //Dproj = ProjKSimplex(Dproj,len,1);
        
        
        for(i=0; i<len; i++) { 
            D[i]=(Xobs[i]-lambda*X[i]);
        
		//Dproj[i]=(Xobs[i]-lambda*X[i])/eta; 
        //if (Dproj[i]<0) {Dproj[i]=0;}
        
        
            if (D[i]<0) Dproj2[i]=0;
            else Dproj2[i]=D[i];
        
        }
    
        //OptSimplex(Dproj,eta, C,len);
        //Dproj2 = ProjKSimplex(Dproj2,len,1);
    
//         for (i=0;i<len;i++){
//             Dproj2[i]=(D[i]-Dproj2[i]);
//             if (Dproj2[i]<0) Dproj2[i]=0;
//      }
        
        
        
        
        primalVal_orig=0;
		for (i=0;i<len; i++)
		{
            primalVal_orig-=Dproj2[i]*Xobs[i];
		}
        
      //  mexPrintf("Iteration: %i, primalval_orig =%1.8f ",iter,primalVal_orig);  
        
        start=clock();
        max_f=0;
        for (i=0;i<len;i++)
        {
            if (Dproj2[i]>max_f)
            	max_f=Dproj2[i];
        }
        // mexPrintf(" eta*max_f =%1.8f ",iter,eta*max_f);  
        
        
        primalVal=primalVal_orig;//+eta*max_f;
        ende=clock();
        time2+=(float) (ende-start);
       
        counter=0;
        primalVal2=0;
        for(j=0; j<cols; j++) 
        {   
            alpha=Dproj2[j];
            for(i=0; i<jcs[j+1]-jcs[j]; i++)
            {  
                primalVal2+=fabs(sr[counter]*( Dproj2[irs[counter]] - alpha));
	            counter++;
            }	  
        }
        
   //     mexPrintf(" primalVal2 =%1.8f \n",iter,primalVal2);  
 
 
        normD = 0; 
        for(i=0; i<len; i++) { 
        	normD+= Dproj2[i]*Dproj2[i];
		}
		Fval = -0.5*normD;//  + C;
   
        primalVal_orig=primalVal_orig+lambda*primalVal2;//+eta*max_f
	    primalVal=primalVal_orig+ 0.5*normD;// +C;
        
        Fval=0.5*Fval;
        primalVal_orig=0.5*primalVal_orig;
        primalVal=0.5*primalVal;
       
        
		// check if we are better than previous best one 
		if (primalVal_orig<primalValBest)
		{
			primalValBest=primalVal_orig;
			dualValBest=Fval;
        
			for(i=0; i<len; i++) { 
				DprojBest[i]=Dproj2[i];
			}
		}
        
        
        ende = clock();
        time4+=(float) (ende-start);
  
   /* 
   
        if (primalValBest<0)
        {
            if((primalVal-Fval)/fabs(Fval)<eps2*100 
            || (- Fval <EPS*100 )) break; 
        } //primalvalbest will always be smaller equal zero
        else if((primalVal-Fval)/fabs(Fval)<eps2 
            || (- Fval <EPS)) break;    // the relative difference 
                                      // between primal and dual
    
    **/
        
// 		if(iter % 10 ==0 || iter<10) { 
//        	 mexPrintf("Iteration: %i, primalval_orig =%1.8f primalval =%1.8f dualval: %1.8f, primalVal-dualval=%1.8f\n",iter,primalVal_orig,primalVal,Fval,(primalVal-Fval)/fabs(Fval));
//         }

        
        if (primalVal<0)
                break;
        else
            MAXITER_temp=MAXITER_temp*2;
        
       
        
                
	}
   
	iter++;
 }

  
//  mexPrintf("time1: %.2f   time2: %.2f  time3: %.2f   time4: %.2f\n",time1 / (float)CLOCKS_PER_SEC,time2 / (float)CLOCKS_PER_SEC,time3 / (float)CLOCKS_PER_SEC,time4 / (float)CLOCKS_PER_SEC);
// mexPrintf("FINAL Iteration: %i,  primalval_orig =%1.8f  primalval =%1.8f dualval: %1.8f, primalVal-dualval=%1.8f\n",iter,primalVal_orig,primalVal,Fval,(primalVal-Fval)/fabs(Fval));

 for(i=0; i<len; i++) { X[i]=Dproj2[i];}
 for(i=0; i<lenrval; i++) { Z[i]=rval[i];}
 PrimalObj[0]=primalVal;
 DualObj[0]=Fval;
 FinalIter[0]=iter;
 for(i=0; i<len; i++) { Xbest[i]=DprojBest[i];}
  PrimalObjBest[0]=primalValBest;
 DualObjBest[0]=dualValBest;
 delete pvalold; delete pval; delete Dproj;delete Dproj2;delete DprojBest;delete D;
	  
	
}





// comparison function for pointers to double
int cmp_pointers(const void *wert1, const void *wert2) {

    double temp = **(double**)wert1 - **(double**)wert2;
    int result;
    if (temp<0) 	result=-1;
    else result= temp>0;

   // printf("wert1 %.2f wert2 %.2f comp =%.2f comp=%d comp=%d\n",**(double**)wert1,**(double**)wert2,temp, (int) temp,result);

return result;
}




// optimizes sum_i max{0,x_i-eta v_i}^2 on the 1-simplex
//double* OptSimplex(double* x,double eta, double D,int len)
void OptSimplex(double* x2,double eta, double D,int len)
{
   // make a copy of x, will later contain max{0,x_i-eta v_i} for the optimal v
   //double * x2 = new double[len];
   //for (int i=0;i<len;i++) {
   //    x2[i]=x[i]; 
   //}
  // printf("unsorted %.2f %.2f %.2f %.2f %.2f \n",x2[0],x2[1],x2[2],x2[3],x2[4]);  
   
   // create an array of pointers to the values of x2 
   double ** xpointers=new double*[len]; 
   for (int i=0;i<len;i++) {
        xpointers[i] = &x2[i];
   }
   
   // sort the array of pointers 
   // now, (int)(xpointers[i]-&x2[0]) will give you the index of the i-th
   // smallest value of x2
   qsort(xpointers, len, sizeof(double), cmp_pointers);	

   /*
   // output results
   printf("sorted ");
   for (int i=0;i<len;i++) {
       printf("%.2f ",*xpointers[i]);
   }
   printf("\n");

   printf("sort indices ");
   for (int i=0;i<len;i++) {
       printf("%d ",(xpointers[i]-&x2[0]));
   }
   printf("\n");


   int * idx= new int[len];
   printf("sort indices ");
   for (int i=0;i<len;i++) {
       idx[i] = (int)(xpointers[i]-&x2[0]);
       printf("%d ",idx[i]);
   }
   printf("\n\n");
*/

   
   // initializations
   int counter=0;
   double sumv=1;
   double * v2 = new double[len];
   for (int i=0;i<len;i++) v2[i]=0;
   
   
   // main loop, we will go from the largest entry to the smallest
   // because of the square term we want to make the large entries small first
   while(counter<len)
   {
       /*
	printf("counter=%d\n",counter);
       printf("x2 %.2f %.2f %.2f %.2f %.2f \n",x2[0],x2[1],x2[2],x2[3],x2[4]);
       printf("v2 %.2f %.2f %.2f %.2f %.2f \n",v2[0],v2[1],v2[2],v2[3],v2[4]);
       printf("sumv %.2f \n",sumv);
       printf("current x_i : %.2f",*xpointers[len-1-counter]);
	*/
	
       // the current entry is larger 0, and we haven't reached the 
       // smallest entry yet
       if ((*xpointers[len-1-counter]>0) && (counter<len-1))
       {
            // compute gap to next smallest entry
            double gap = *xpointers[len-1-counter]-*xpointers[len-1-counter-1];
            //printf("gap : %.2f\n",gap);

            // if the gap is larger 0, we want to reduce the current 
            // entry to the next smallest entry; note that all entries 
            // which we have already considered have the same value than 
            // the current entry
            if (gap>0)
            {
                //printf("counter =%d gap *(counter+1) : %.2f",counter,gap*(counter+1));

                // if there is still "enough v left"
                if (gap*(counter+1)<sumv*eta)
                {
                    for (int j=0;j<=counter;j++)
                    {	
                        *xpointers[len-1-j]=*xpointers[len-1-counter-1];
                        *(xpointers[len-1-j]-&x2[0]+&v2[0])+=gap/eta;
                    }
                    sumv-=gap*(counter+1)/eta;
                }
                // if not we reduce as much as possible
                else
                {
                    for (int j=0;j<=counter;j++)
                    {	
                        *xpointers[len-1-j]-=sumv*eta/(counter+1);
                        *(xpointers[len-1-j]-&x2[0]+&v2[0])+=sumv/(counter+1);
                    }
                    sumv=0;
                }
            }
            // if the gap is 0, we can go to the next entry
            //else
            //{
            //    printf("Skipping\n");
            //}
		}
        // if we have reached the smallest entry, all entries must have the 
        // same value; if the current entry is negative, all previous entries,
        // and hence all entries also have to be negative; in both cases 
        // we can now evenly distribute the "rest" of v on all components
       else if (*xpointers[len-1-counter]>sumv*eta/len)
		{
			for (int j=0;j<=len-1;j++)
			{	
				*xpointers[len-1-j]-=sumv*eta/len;
				*(xpointers[len-1-j]-&x2[0]+&v2[0])+=sumv/len;
			}
			sumv=0;

		}	
       else
       {
           for (int j=0;j<=len-1;j++)
			{	
				*xpointers[len-1-j]=0;
				*(xpointers[len-1-j]-&x2[0]+&v2[0])+=sumv/len;
			}
			sumv=0;
       }
	counter=counter+1;
	//printf("\n\n");
    // quit the loop if we don't have any v left
	if (sumv==0) break;
  }
   
  // display final results
  //printf("x2 %.2f %.2f %.2f %.2f %.2f \n",x2[0],x2[1],x2[2],x2[3],x2[4]);  
  //printf("v2 %.2f %.2f %.2f %.2f %.2f \n",v2[0],v2[1],v2[2],v2[3],v2[4]);  
  //printf("sumv %.2f \n",sumv);  

  delete xpointers; delete v2;
  //return x2;
}


double* ProjKSimplex(double* f,int len, int K)
{
  int i;
  //double* xnew = new double[len];
  int* IX = new int[len];
  double sumxnew = 0; double sumxnew2;
  int counter=len; int counter2=0;
  for(i=0; i<len; i++)
   { IX[i]=i; sumxnew+=f[i];}
  int iteration=0;
  while(iteration<10000)
  {
    counter2=0; sumxnew2=0;
    for(i=0;i<counter; i++)
	{
	  f[IX[i]] = f[IX[i]] - (sumxnew-K)/counter;
	  if(f[IX[i]]>0)
	   { IX[counter2]=IX[i]; counter2++; sumxnew2+=f[IX[i]]; }
	  else
	   f[IX[i]]=0;
	}
	if(counter2==counter) break;
	counter=counter2; sumxnew=sumxnew2;
	iteration++;
  }
  if(iteration==10000)
	  mexPrintf("Bug in Projection: %i, Fval: %i\n",counter,counter2);
  delete IX;
  return f;
  /*x = f;  IX=1:num;  
  while(cond)
   xnew = x;
   xnew(IX) = x(IX) - (sum(x)-k)/length(IX);
   if(sum(xnew>=0)==num)
    Projf=xnew; break;
   else
    %ixx = find(xnew<0);
    %IX = setdiff(IX,ixx);
    IX = find(xnew>0);
    x = max(xnew,0);
   end
  end*/
}

