%
% (C)2012-13 Syama Sundar Rangapuram, Thomas Buehler and Matthias Hein 
% Machine Learning Group, Saarland University, Germany
% http://www.ml.uni-saarland.de

% M denotes the skill matrix
% Here is the skill order: [ DB DM AI T ]
% Set initialization here: (To get the LP bound set initialization = 2)

%initialization = 0; % No initialization
%initialization = 1; % Initialize with the team found by mdAlk; result of mdAlk is also saved here!
%initialization = 2; % Initialize with the team found from LP relaxation; result of LP relaxation is also saved here!

initialization = input('Enter the initialization option for Forte (0-2): '); 

load dataForQuantitativeExperiments

options.MAXITER = 1000;
options.perturbation = true;
options.tol_score = 1e-4;

teams_forte = zeros(n, length(ks), nTrials);
scores_forte = zeros(length(ks), nTrials);
sizes_forte = zeros(length(ks), nTrials);
times_forte = zeros(length(ks), nTrials); 

initial_str = '';
switch initialization
    case 1,            
        teams_mdAlk = zeros(n, length(ks), nTrials);
        scores_mdAlk = zeros(length(ks), nTrials);
        sizes_mdAlk = zeros(length(ks), nTrials);
        times_mdAlk = zeros(length(ks), nTrials); 
    case 2,
        addpath('/opt/ibm/ILOG/CPLEX_Studio124/cplex/matlab/');
        mhalf = 0.5*sum(sum(W>0));
        ys_lp = zeros(n, length(ks), nTrials);
        fs_lp = zeros(n, length(ks), nTrials);
        alphas_lp = zeros(mhalf, length(ks), nTrials);       
        tvars_lp = zeros(length(ks), nTrials);        
        objectives_lp = zeros(length(ks), nTrials);
        statuses_lp = -inf*ones(length(ks), nTrials);        
        times_lp = zeros(length(ks), nTrials); 
        times_lp_total = zeros(length(ks), nTrials); 
        times_lp_set_time = zeros(length(ks), nTrials);             

        teams_lp = zeros(n, length(ks), nTrials);
        scores_lp = zeros(length(ks), nTrials);
        sizes_lp = zeros(length(ks), nTrials);
end

for t=1:10
    for i=1:length(ks)
        lbs = lbs_all(:, i, t);
        lbs = lbs';                

        switch initialization
            case 0, [teams_forte(:,i,t), scores_forte(i,t), times_forte(i,t)] = forte(W, M, lbs, sparse(n,1), 0, g, [], sparse(n,n), 0, options);

            case 1, time_mdAlk_start = tic;
                    [teams_mdAlk(:, i, t), scores_mdAlk(i, t)] = mdAlk(W, g, M, lbs); 
                    times_mdAlk(i, t) = toc(time_mdAlk_start);
                    % find other statistics
                    sizes_mdAlk(i, t) = sum(teams_mdAlk(:, i, t));

                    options.initialization = teams_mdAlk(:, i, t);
                    [teams_forte(:,i,t), scores_forte(i,t), times_forte(i,t)] = forte(W, M, lbs, sparse(n,1), 0, g, [], sparse(n,n), 0, options); 
                    times_forte(i,t) = times_forte(i,t) + times_mdAlk(i, t);

            case 2, 
                    time_lp_start = tic;              
                    [y, X, f, alpha, tvar, objective, status, status_text, lp_time, lp_set_time] = gdsp_lp_relax_cplex(W,0,lbs',g,sparse(n,1), M, [], sparse(n,n), 0);
                    times_lp_total(i, t) = toc(time_lp_start);                

                    ys_lp(:, i, t) = y; fs_lp(:, i, t) = f; alphas_lp(:, i, t) = alpha; tvars_lp(i, t) = tvar; statuses_lp(i, t) = status;
                    objectives_lp(i, t) = objective; times_lp(i, t) = lp_time; times_lp_set_time(i, t) = lp_set_time;

                    team = opt_thresh_forte_hard_cnstrs(W, ys_lp(:, i, t), g, lbs, 0, M, sparse(n, 1), 0, 0, W(:, []), sparse(n, n));
                    teams_lp(:, i, t) = team;
                    %cc_score = compute_density(W, g, team>0);
                    cc_score = generalized_density(W, g, team);
                    scores_lp(i, t) = cc_score;
                    sizes_lp(i, t) = sum(team);

                    options.initialization = teams_lp(:, i, t);
                    [teams_forte(:,i,t), scores_forte(i,t), times_forte(i,t)] = forte(W, M, lbs, sparse(n,1), 0, g, [], sparse(n,n), 0, options);                    
        end
        sizes_forte(i, t) = sum(teams_forte(:,i,t));


    end
end

% Find the averages... 
scores_forte_avg = mean(scores_forte, 2);
sizes_forte_avg = mean(sizes_forte, 2);
times_forte_avg = mean(times_forte, 2);

dataset_results = 'quantitativeExperiments';

switch initialization
    case 1, scores_mdAlk_avg = mean(scores_mdAlk, 2);
            sizes_mdAlk_avg = mean(sizes_mdAlk, 2);
            times_mdAlk_avg = mean(times_mdAlk, 2);
            dataset_results = [dataset_results '_mdAlk_plus'];

    case 2, scores_lp_avg = mean(scores_lp, 2);
            sizes_lp_avg = mean(sizes_lp, 2);
            times_lp_avg = mean(times_lp, 2);
            dataset_results = [dataset_results '_lp_plus'];
end

dataset_results = [dataset_results '_forte'];
save(dataset_results);

